#include <iostream>
#include <string>
#include <list>
#include <fstream>
#include <WordcatClasses.h>

using namespace std;

list<Node> CreateNode(list<Node> Nodes, string nodename, int depth)
{
	list<Node>::iterator q = Nodes.begin();
	bool done = false;

	// do while loop inserts nodes in order if it doesn't exist
	do
	{	if (q == Nodes.end())  // Checks if you've reached the end and not done - if so create the node in last place
		{	Nodes.push_back(Node(nodename, depth));
			done = true;
		}
		else if ((q != Nodes.end()) && (q->GetName() == nodename))// if name is the same, don't make a new one.  If depth is different, wait till next round to make one.
		{	if(q->GetDepth() == depth)
				done = true;
			else if (q->GetDepth() != depth)
			{	list<Node> temp;
				temp.push_back(Node(nodename, depth));
				Nodes.splice(q, temp);
				done = true;
			}
		}
		else if ((q != Nodes.end()) && (q->GetName() > nodename))  // If it's not the end, and the Node's name is higher that than the one you wanna create, insert it
		{	list<Node> temp;
			temp.push_back(Node(nodename, depth));
			Nodes.splice(q, temp);
			done = true;
		}
			++q;
	}	while (!done);
	
// do while loop done

//	cout << ".";

	return Nodes;
}

list<SuccConn> CreateLink(list<SuccConn> Links, string prev, string current, float weight)
{
	list<SuccConn>::iterator p;

	if (prev.empty())
		return Links;

	if (Links.empty())  // if there are no links, make one
			Links.push_back(SuccConn(prev, current, weight));

	else  // if the link doesn't exist, create it in the right order.  If it does, strengthen it.
	{	p = Links.begin();
		bool done = false;

		// do while loop inserts links in order
		do
		{	if (p == Links.end())  // Checks if you've reached the end and not done - if so create the node in last place
			{	Links.push_back(SuccConn(prev, current, weight));
				done = true;
			}
			else if ((p != Links.end()) && (p->GetPred() == prev)) // if not at end, and first link element is same, check second.
			{	if (p->GetSucc() == current) // if second is the same, it already exists - strengthen the link
				{	p->IncreaseWeight();
					done = true;
				}
				else if (p->GetSucc() > current) // insert link.
				{	list<SuccConn> temp;
					temp.push_back(SuccConn(prev, current, weight));
					Links.splice(p, temp);
					done = true;
				}
			}
			else if ((p != Links.end()) && (p->GetPred() > prev)) // if not at end, and first link element is smaller, insert it
				{	list<SuccConn> temp;
					temp.push_back(SuccConn(prev, current, weight));
					Links.splice(p, temp);
					done = true;
				}
			++p;
		}	while (!done);
		// do while loop done
	}

//	cout << "-";

	return Links;
}


list<FamConn> InsertFamLink(list<FamConn> Links, string prev, string current, float num)
{
	list<FamConn>::iterator p;
	float weight;

// determine Famlink Weight
	if (num == 1)
		weight = .2;
	else if (num > 1)
		weight = (1/num);

// Make the link
	if (Links.empty())  // if there are no links, make one
		Links.push_back(FamConn(prev, current, weight));
	
	else// if the link doesn't exist, create it in the right order.
	{	p = Links.begin();
		bool done = false;

		// do while loop inserts links in order
		do
		{	if (p == Links.end())  // Checks if you've reached the end and not done - if so create the node in last place
			{	Links.push_back(FamConn(prev, current, weight));
				done = true;
			}
			else if ((p != Links.end()) && (p->GetPred() == prev)) // if not at end, and first link element is same, check second.
			{	if (p->GetSucc() == current) // if second is the same, it already exists
					done = true;
				else if (p->GetSucc() > current) // insert link.
				{	list<FamConn> temp;
					temp.push_back(FamConn(prev, current, weight));
					Links.splice(p, temp);
					done = true;
				}
			}
			else if ((p != Links.end()) && (p->GetPred() > prev)) // if not at end, and first link element is smaller, insert it
				{	list<FamConn> temp;
					temp.push_back(FamConn(prev, current, weight));
					Links.splice(p, temp);
					done = true;
				}
			++p;
		}	while (!done);
		// do while loop done
	}

	return Links;
}


list<FamConn> CreateFam(list<Node> Nodes)
{
	list<FamConn> FamLinks;
	list<FamConn>::iterator p;
	list<Node>::iterator q;

	for (q = Nodes.begin(); q != Nodes.end(); ++q)
	{	Node node = *q;
//		if (node.GetDepth() == 2)  // Creates links between 1 and (2&3)
//		{	string name = node.GetName();
//
//			int number;	
//
//			if (node.GetDepth() == 1)
//				number = (name.size()*2)+1;
//			else if (node.GetDepth() == 2)
//				number = 2;
//
//			for (int i = 0; i != name.size(); ++i)
//			{	string prev = "";
//				prev += name[i];
//				if (node.GetDepth() == 2)
//					FamLinks = InsertFamLink(FamLinks, prev, name, number);
//				if (node.GetDepth() == 3)
//					FamLinks = InsertFamLink(FamLinks, name, prev, 1);
//			}
//		}

		// Create Family Links from 2 - 3
		if (node.GetDepth() == 3)
		{	string longname = "*" + node.GetName() + "*";
			string shortname = node.GetName();
			int number = shortname.size()+1;
			shortname = "3"+shortname;

			string prev = "";
			prev = prev + longname[0] + longname [1];
			FamLinks = InsertFamLink(FamLinks, prev, shortname, number);
			FamLinks = InsertFamLink(FamLinks, shortname, prev, 1);

			for (int i = 2; i != longname.size(); ++i)
			{	prev = prev[1];
				prev += longname[i];
				FamLinks = InsertFamLink(FamLinks, prev, shortname, number);
				FamLinks = InsertFamLink(FamLinks, shortname, prev, 1);
			}
		}
		cout << ".";
	}

	return FamLinks;
}

int CreateNetwork()
{
	string prev = "*", current, word ="", line;
	list<Node> Nodes;
	list<Node>::iterator q;
	list<SuccConn> Links;
	list<SuccConn>::iterator p;
	list<FamConn> Family;
	list<FamConn>::iterator r;

	bool stop1, stop2;
	int i;

	Nodes.push_back(Node(prev, 1));

//********************** Read File and Create Nodes & SuccConn Network *********************************

// open text to be analyzed
	ifstream in("Text.txt");
	if(!in)
	{	cout << "Cannot open input file.\n";
		return 0;
	}

	cout << "Working";

//start while loop to read file
	while (!in.eof())
	{
		cout << ".";
		in >> line;

	//start for loop to read in one line
		for (i = 0; i != line.size(); ++i)
		{	

			stop1 = (line.size()>i+1);
			stop2 = (line.size()>i+2);

//  Depth 1 -> Depth 1 Nodes and Links
			current = line[i];

			CreateNode(Nodes, current, 1);
			Links = CreateLink(Links, prev, current, 0.05);
		
//  Check for Word
			if (current == "*")
			{	Nodes = CreateNode (Nodes, word, 3);
				word = "";
			}
			else if (current != "*")
				word += current;

//  Depth 1 -> Depth 2 Nodes and Links

			if (stop1)
			{	current += line[i+1];

				Nodes = CreateNode(Nodes, current, 2);
				if(current[0] != '*')
					Links = CreateLink(Links, prev, current, 0.05);
			

//  Depth 2 -> Depth 1 Nodes and Links

				prev += current[0];
				current = current[1];

				Nodes = CreateNode(Nodes, prev, 2);
	
				Nodes = CreateNode(Nodes, current, 1);
//		Links = CreateLink(Links, prev, current, 0.05);
			}

//  Depth 2 -> Depth 2 Nodes and Links
			if (stop2)
			{	current += line[i+2];

				Nodes = CreateNode(Nodes, current, 2);
//				Links = CreateLink(Links, prev, current, 0.05);
			}

			if (stop1)
				prev = prev[1];
			else
				prev = current;
			

		}
	// end for loop to read one line

	}
// end while loop to read file

	in.close();

//******************************** Finished Reading File and Creating Nodes and SuccConn Network ***************

	cout << "|";

//******************************** Create Depth 3 Inhibitory Links ***************

//	list<string> wordlist;
//	list<string>::iterator wlcounter1, wlcounter2;
//
//	for (q = Nodes.begin(); q != Nodes.end(); ++q)
//	{
//		if(q->GetDepth() == 3)
//			wordlist.push_back(q->GetName());
//	}
//
//	for (wlcounter1 = wordlist.begin(); wlcounter1 != wordlist.end(); ++wlcounter1)
//	{	for (wlcounter2 = wordlist.begin(); wlcounter2 != wordlist.end(); ++wlcounter2)
//			if (wlcounter1 != wlcounter2)
//				Links = CreateLink(Links, *wlcounter1, *wlcounter2, -.1);
//		cout << ".";
//	}

//******************************** Create FamConn Network ************************

	cout << "|";
	Family = CreateFam(Nodes);

//******************************** Finished Creating FamConn Network *************
// open output file
	ofstream out("Network.txt");
	if(!out)
	{	cout << "Cannot open output file.\n";
		return 0;
	}

	cout << endl << "Nodes: " << Nodes.size() << endl;
	out << "Nodes: " << Nodes.size() << endl;

	for (q = Nodes.begin(); q != Nodes.end(); ++q)
		out << q->GetName() << " " << q->GetDepth() << endl;

	cout << "Links: " << Links.size() << endl;
	out << "Links: " << Links.size() << endl;

	for (p = Links.begin(); p != Links.end(); ++p)
		out << p->GetPred() << " " << (p->GetWeight()) << " " << p->GetSucc() << endl;

	cout << "Family: " << Family.size() << endl; 
	out << "Family: " << Family.size() << " end\n";

	for (r = Family.begin(); r != Family.end(); ++r)
		out << r->GetPred() << " " << r->GetSucc() << " " << r->GetWeight() << endl;



	out.close();

	cout << "Yes! Yes! I worked!\n";

	return 1;
}
